/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : Dec. 2006

 File          : LIN_FRAME_SLAVE_LINSRTGND.c

 Hardware      : Applicable to ADuC703x

 Description   : LIN Slave code for 
				 The debugger cannot be used when the part is placed in power down mode.
		
*********************************************************************/

#include <ADuC7038.h>

void LIN_INIT(void);
void My_FIQ_Function(void);
void My_IRQ_Function(void);

void delay(int);
int LIN_status;
int iFIQ_Source,iIRQSIG, array_counter = 0;
int FRAME_ERROR = 0;
unsigned char LIN_Data[12];
int POWERED_UP=1;

	unsigned long it1=1234, it2=4321;

int main()
{
	iFIQ_Source = IRQSIG;			// Save a temporary copy of IRQSTA	
	LIN_INIT();

    GPCON = 0x00000000;	//   all GPIOs
    GPDAT = 0x3F3F0000;	// Set all bits to outputs	- high

	FIQ = My_FIQ_Function; 	// Initialise Interrupt Routine	 for LIN
	FIQEN = 0x20;			// Enable LIN interrupt

	IRQ = My_IRQ_Function; 	// Initialise Interrupt Routine	 for T1
	IRQEN = 0x08;

	T1LD = 0x1F40;   		// full scale 524 sec	/16 = 2 sec	  - lpck
	T1CON = 0x0D;			// 16 prescaler .. 2 sec time, low p clk
	SPICON=0;
	ADC0CON=0;
	ADC1CON=0;

	while(1)
										
	{
	}				// Infinite Loop
}

void My_IRQ_Function() 
{
	iIRQSIG = IRQSIG;
	iFIQ_Source = IRQSTA;			// Save a temporary copy of IRQSTA		
 
 if (POWERED_UP)
 	{
    POWERED_UP = 0;					// now powered down
 	T1CLRI = 0x01;					// restart Timer 1
	POWKEY0 = 0x01;		 			// Put all peripherals, PLL and Core Asleep
	POWCON  = 0x00;					// default value is 79	- 10MHz
	POWKEY1 = 0xF4;					// 
	it1 = it1*it2;					// Dummy cycle to ensure correct power down
	}
	else
	{
	POWKEY0 = 0x01;		 			// Put all peripherals, PLL and Core Asleep
	POWCON  = 0x79;					// default value is 79	- 10MHz
	POWKEY1 = 0xF4;					// 
	it1 = it1*it2;					// Dummy cycle to ensure correct power down
	T1LD = 0x1F40;   		// full scale 524 sec	/16 = 2 sec	  - lpck
	T1CON = 0x0D;			// 16 prescaler .. 2 sec time, low p clk
     POWERED_UP = 1;	
	}

	return ;
}

void My_FIQ_Function() 
{
   	LIN_status = LINSTA;		// 0000 0011 1000 1001  - 389	- Frame error cleared
			
 	if (LIN_status & 0x08)		// Break error has occurred.
		{
		POWKEY0 = 0x01;		 			// Put all peripherals, PLL and Core Asleep
		POWCON  = 0x79;					// default value is 79	- 10MHz
		POWKEY1 = 0xF4;					// 
		it1 = it1*it2;					// Dummy cycle to ensure correct power down

		T1CON = 0x0D;			// 16 prescaler .. 2 sec time, low p clk
		T1LD = 0x1F40;   		// full scale 524 sec	/16 = 2 sec	  - lpck
		POWERED_UP	=1;
		GPCLR = 0x0020000;		// GPIO1	
		delay(50);
		GPSET = 0x0020000;		

//		array_counter = 0; 		// Break detected - So new data coming
//		while ((LIN_status & 0x200)	==0)
//			LIN_status = LINSTA;
//		if (LIN_status & 0x200)		// MAX Break Exceeded 
//			{
//			GPCLR = 0x0010000;		// GPIO0	
//			delay(50);
//			GPSET = 0x0010000;		
//			}
//			LINCON = 0x880;		// Rx mode
		}


	if (LIN_status & 0x01)		// Rx.
		{
		LIN_Data[array_counter] = LINDAT;		// clears the interrupt
		GPCLR = 0x0020000;		
		delay(50);
		GPSET = 0x0020000;		
//		array_counter++;
		}
//		if 	(array_counter == 1)
//			{					// Received the PID
//			LINCON = 0x881;		// Tx mode
//			}

//							//10 LIN wake up interrupt
//	1						//9 Break time max
//	1						//8 PID parity Error
//	1						//7 Checksum Match
//	1 PID Stop/Start		//6 Frame Error.  ------------***
//							//5 Negative edge max error interrupt  maskable
//	1						//4 LIN collision detect Interrupt  maskable		
//	1						//3 Break Receive Interrupt  maskable
//						 	//2 Transmit Complete Interrupt  maskable
//							//1 Transmit Ready Interrupt  maskable
//  1						//0 Receive Ready Interrupt
							// Clear interrupt

	if (LIN_status & 0x02)		// Tx 
		{
		GPCLR = 0x0020000;		// GPIO1
		delay(50);
		GPSET = 0x0020000;		
		}
		
	if (LIN_status & 0x10)		// Collision   or NEG Edge
		{
		GPCLR = 0x0040000;		// GPIO2
 		delay(50);
		GPSET = 0x0040000;
		LINCON = 0x0;					//  Rx mode
		LIN_INIT();	
		}
			
	if (LIN_status & 0x80)		// Checksum Match
		{
//		GPCLR = 0x0080000;		
		delay(50);
//		GPSET = 0x0080000;
		LINCON = 0x880;	
		}	  
			
	if (LIN_status & 0x040)		// Frame error has occurred.
		{
		GPCLR = 0x0040000;		// GPIO3
		delay(50);
		GPSET = 0x0040000;
		LIN_INIT();	
		}
 
	 return;

}


void delay (int length)
{
	while (length >=0)
    	length--;
}

void LIN_INIT(void )
{
int  HVstatus;
	do										
	{
		HVDAT = 0x00; 						// Enable Normal LIN TX mode / reenable short circuit
		HVCON = 0x08; 						// Write to Config
		do
		{
			HVstatus = HVCON;
		}
		while(HVstatus & 0x1); 				//  command finished
	}
	while (!(HVstatus & 0x4));				// transmit command 

// LINWU = 0xfffff;							// Wake Up with INWU also
// LINBK = 5500;						// Set number of 128kHz periods to generate a Break Interrupt
//LINBK = 0x7700;							// 0x46 / 128kHz ~ 492us which is just over 11 TBits.
											// According to LIN V2.0, a slave with a clock accuracy of <2% may
											// use a break detection of 9.5 bit, otherwise a break detection of
											// 11 bits is required
											// This is for 19200bps
	LINCON = 0x880;						// Enable LIN , checksum, BREAK=0 - enabled.
//	LINDAT = 0xF0;
}

